// Copyright (C) 2014 by Karlsruher Institut fuer Technologie (KIT), Institut fuer Mess- und Regelungstechnik
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of arDroneManual/Autonomous.
// 
// arDroneManual/Autonomous is free software: you can redistribute it and/or modify it under the terms of the GNU 
// General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your 
// option) any later version.
// 
// arDroneManual/Autonomous is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
// 
// You should have received a copy of the GNU General Public License along with arDroneManual/Autonomous. If not, see 
// <http://www.gnu.org/licenses/>.


// closed-loop controllers
// =======================

#pragma once

#include "hawaii/common/timer.h"
#include <limits>
#include <cmath>

// PID controller with anti-wind-up
class ControllerPID {
	
	// optional full initialization with PID-parameters and actuator limits
	public:
	ControllerPID( const double kPArg   = 1.0,
	               const double kIArg   = 0.0,
	               const double kDArg   = 0.0,
	               const double uMaxArg = std::numeric_limits< double >::max(),
	               const double uMinArg = std::numeric_limits< double >::min() ) ;
	
	// public parameters
	public:
	double kP, kI, kD ; // controller constants
	double uMax, uMin ; // actuator limits for anti-wind-up
	
	// get actuator command from error, which is target minus actual control value
	public:
	double operator ()( const double error,
	                          double errorDeriv = NAN ) ;
	
	// temporarily disable and re-enable the integrator e.g. before/after actuator is ineffective
	public:
	void integratorDisable() ;
	void integratorEnable()  ;
	protected:
	bool integratorActive ; 
	
	// internal state, reset it
	public:
	void reset() ;
	protected:
	double errorIntegral ; // integral of previous errors
	double errorPrevious ; // error at last call
	hawaii::Timer timer ; // time since last call
	
} ; // class "ControllerPID"
