# Copyright (C) 2014 by Karlsruher Institut fuer Technologie (KIT), Institut fuer Mess- und Regelungstechnik
# 
# Author: Benjamin Ranft (benjamin.ranft@web.de)
# 
# This file is part of arDroneManual/Autonomous.
# 
# arDroneManual/Autonomous is free software: you can redistribute it and/or modify it under the terms of the GNU 
# General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your 
# option) any later version.
# 
# arDroneManual/Autonomous is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
# the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
# more details.
# 
# You should have received a copy of the GNU General Public License along with arDroneManual/Autonomous. If not, see 
# <http://www.gnu.org/licenses/>.


# script to generate clozes ("Lueckentexte") from annotated source code
# =====================================================================

# display help and abort if 1st argument is neither "mtp" nor "rvmrt"
if [ ! "$1" = "mtp" ] && [ ! "$1" = "rvmrt" ] ; then
echo "missing lab argument: call either \"clozer.sh mtp\" or \"clozer.sh rvmrt\""
exit 1
fi

# path of annotated complete code and to-be-generated clozes
SRC=/home/administrator/mobileRoboterplattform/code
DST=/home/praktikum/Desktop/code
echo ""
echo "generating clozes from "$SRC" at "$DST

# clear existing clozes path
echo ""
echo "clearing "$DST
rm -rf $DST/*
mkdir -p $DST

# copy only neccessary parts of base projects which are not to be modified
echo "copying base files"
cp    $SRC/COPYING             $DST/
cp    $SRC/INSTALL             $DST/
cp    $SRC/INSTALL_MRT         $DST/
cp    $SRC/makeClean.sh        $DST/
cp    $SRC/README              $DST/
cp    $SRC/ros.sh              $DST/
echo "copying \"clozer\" header"
mkdir $DST/clozer
cp -r $SRC/clozer/clozer.h     $DST/clozer/
echo "building and copying \"libHawaii\" headers and binaries"
cd $SRC/libHawaii && make > /dev/null
mkdir $DST/libHawaii
cp -r $SRC/libHawaii/inc       $DST/libHawaii/
cp -r $SRC/libHawaii/lib       $DST/libHawaii/
cp -d $SRC/libHawaii/makefile  $DST/libHawaii/
cp -d $SRC/libHawaii/config.mk $DST/libHawaii/
cp -d $SRC/libHawaii/rules.mk  $DST/libHawaii/
echo "building and copying \"arDroneManual\" headers and binaries"
cd $SRC/arDroneManual && make > /dev/null
mkdir $DST/arDroneManual
cp -r $SRC/arDroneManual/bin   $DST/arDroneManual/
cp -r $SRC/arDroneManual/data  $DST/arDroneManual/
cp -r $SRC/arDroneManual/inc   $DST/arDroneManual/

# completely copy projects for students to work in
echo "copying \"arDroneAutonomous\" sources"
cp -r $SRC/arDroneAutonomous   $DST/
rm -rf $DST/arDroneAutonomous/obj
rm -f  $DST/arDroneAutonomous/bin/*
echo "copying \"offLineDetection\" sources"
cp -r $SRC/offLineDetection    $DST/
rm -rf $DST/offLineDetection/obj
rm -f  $DST/offLineDetection/bin/*

# create clozes by processing hints within all source code files
# developer note #1: "clozer.h" must be excluded in order to not break it.
# developer note #2: "offLineDetection/src/lineDetection.cpp" must be excluded because otherwise it would not be a 
#                    symlink to "arDroneAutonomous/src/lineDetection.cpp" any more.
echo ""
echo "modifying files..."
if [ "$1" = "mtp" ] ; then
echo "modifying files for MTP..."
FILES=$(find $DST/ -type f -name "*.h" ! -name "clozer.h" -o -name "*.hpp" -o -name "*.c" -o -name "*.cpp" ! -wholename "*/offLineDetection/src/lineDetection.cpp")
fi
if [ "$1" = "rvmrt" ] ; then
echo "modifying files for RVMRT..."
FILES=$(find $DST/ -type f -name "*.h" ! -name "clozer.h" -o -name "*.hpp" -o -name "*.c" -o -name "*.cpp" ! -wholename "*/offLineDetection/src/lineDetection.cpp" ! -wholename "*/arDroneAutonomous/src/appFollowLine.cpp")
fi
for FILE in $FILES
do
  
  # print file name with only relative path
  echo "- "${FILE#$DST/}
  
  # before: int foo = 5 ; // STUD_TODO tune this parameter
  # after:  int foo = 5 ; // TODO tune this parameter
  sed -i 's:// STUD_TODO:// TODO:' $FILE
  
  # before: bar = foo * 2 ; // STUD_REMOVE set foo twice as big as foo
  # after:  // TODO set foo twice as big as foo
  sed -i 's:[A-Za-z].*// STUD_REMOVE:// TODO:' $FILE
  
  # before: func( STUD_NAN( 0.5 ), 3 ) ; // STUD_TODO set first argument
  # after:  func( NAN, 3 ) ; // TODO set first argument
  sed -i 's:STUD_NAN([^)]*):NAN:' $FILE
  sed -i 's:STUD_ZERO([^)]*):0:' $FILE
  sed -i 's:STUD_MINUSONE([^)]*):-1:' $FILE
  
  # before: bar = foo * 2 ; // STUD_SET_ZERO set foo twice as big as foo
  # after:  bar = 0 ; // TODO set foo twice as big as foo
  sed -i 's:=[^=]*// STUD_SET_NAN:= NAN ; // TODO:' $FILE
  sed -i 's:=[^=]*// STUD_SET_ZERO:= 0 ; // TODO:' $FILE
  sed -i 's:=[^=]*// STUD_SET_MINUSONE:= -1 ; // TODO:' $FILE
  sed -i 's:=[^=]*// STUD_SET_NOTHING:; // TODO:' $FILE
  
  # before: func( 0.5, 3 ) ; // STUD_FUNCTION find suitable function
  # after:  DUMMY( 0.5, 3 ) ; // TODO find suitable function
  # This is not suitable for functions or methods with return values. See below for always returning "NAN".
  sed -i "/.*\/\/ STUD_FUNCTION.*/{ s:[A-Za-z][^(]*(:DUMMY(: }" $FILE
  sed -i "s:// STUD_FUNCTION:// TODO:" $FILE
  
  # before: bar = STUD_FUNCTION_NAN( atan )( foo ) * wak ;
  # after:  bar = DUMMY_NAN( foo ) * wak ;
  sed -i "s:STUD_FUNCTION_NAN([^)]*):DUMMY_NAN:" $FILE
  
done
echo "done modifying files"

# build the modified programs
echo ""
echo "building \"arDroneAutonomous\" (warnings may be ignored)"
cd $DST/arDroneAutonomous && make
echo ""
echo "building \"offLineDetection\" (warnings may be ignored)"
cd $DST/offLineDetection  && make

# grant students write access to copied data
echo "granting write access"
chmod -R o+w $DST

# done
echo ""
echo "done creating clozes"
echo ""
exit 0

