// Copyright (C) 2014 by Karlsruher Institut fuer Technologie (KIT), Institut fuer Mess- und Regelungstechnik
// 
// Authors: Benjamin Ranft (benjamin.ranft@web.de), Jacques Maylaender
// 
// This file is part of arDroneManual/Autonomous.
// 
// arDroneManual/Autonomous is free software: you can redistribute it and/or modify it under the terms of the GNU 
// General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your 
// option) any later version.
// 
// arDroneManual/Autonomous is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
// 
// You should have received a copy of the GNU General Public License along with arDroneManual/Autonomous. If not, see 
// <http://www.gnu.org/licenses/>.


// line detection
// ==============

// note: "arDroneAutonomous/src/lineDetection.h" and "offLineDetection/src/lineDetection.h" are symlinked, i.e. 
//       modifications occur in both at the same time.

#pragma once

#include <opencv2/core/core.hpp>

// 3rd order polynomial coefficients: For convenience, this type includes a method to compute column "u" from row "v" 
//                                    according to "u = c0*v³ + c1*v² + c2*v + c3" 
struct Coeffs : public cv::Vec4d {
	double u( double v ) const {
		return ( this->val[ 0 ] * v*v*v
		       + this->val[ 1 ] * v*v
		       + this->val[ 2 ] * v
		       + this->val[ 3 ]         ) ;
	}
	Coeffs() : cv::Vec4d( NAN, NAN, NAN, NAN ) {}
} ;

// estimate such coefficients from an image of a roughly vertical red line
Coeffs detectLine( const cv::Mat image ) ;

